@@ -3,7 +3,6 @@ using System.Collections;
 
 namespace FSpot {
 	public class DirectoryAdaptor : GroupAdaptor {
-		public PhotoQuery query;
 		System.Collections.DictionaryEntry [] dirs;
 
 		// FIXME store the Photo.Id list here not just the count
@@ -37,15 +36,7 @@ namespace FSpot {
 
 			Console.WriteLine ("Selected Path {0}", dirs [group].Key);
 	
-			int item = 0;
-			int i = 0;
-			while (i < query.Count) {
-				if (((Photo)(query [i])).DirectoryPath == (string)dirs [group].Key) {
-					item = i;
-					break;
-				}
-				i++;
-			}
+			int item = LookupItem (group);
 
 			if (GlassSet != null)
 				GlassSet (this, item);
@@ -71,17 +62,11 @@ namespace FSpot {
 			return ((DirectoryAdaptor.Group)dirs [item].Value).Count;
 		}	
 
-		private void HandleChanged (FSpot.IBrowsableCollection query)
-		{
-			Console.WriteLine ("Reloading directory");
-			Reload ();
-		}
-		
 		public override event ChangedHandler Changed;
-		public override void Reload () 
+		protected override void Reload () 
 		{
 			System.Collections.Hashtable ht = new System.Collections.Hashtable ();
-			Photo [] photos = query.Store.Query (null, null);
+			Photo [] photos = query.Store.Query ((Tag [])null, null, null, null);
 			
 			foreach (Photo p in photos) {
 				if (ht.Contains (p.DirectoryPath)) {
@@ -98,6 +83,11 @@ namespace FSpot {
 			Array.Sort (dirs, new DirectoryAdaptor.Group ());
 			Array.Sort (query.Photos, new Photo.CompareDirectory ());
 			
+			if (!order_ascending) {
+				Array.Reverse (dirs);
+				Array.Reverse (query.Photos);
+			}
+			
 			if (Changed != null)
 				Changed (this);
 		}
@@ -117,17 +107,25 @@ namespace FSpot {
 			return 0;
 		}
 
-		public override int GetInitialPosition()
+		public override FSpot.IBrowsableItem PhotoFromIndex (int item) 
 		{
-			// FIXME not truly implemented
+			return query.Items [LookupItem (item)];
+		}
+
+		private int LookupItem (int group)
+		{
+			int i = 0;
+			while (i < query.Count) {
+				if (((Photo)(query [i])).DirectoryPath == (string)dirs [group].Key) {
+					return i;
+				}
+				i++;
+			}
 			return 0;
 		}
 
-		public DirectoryAdaptor (PhotoQuery query) {
-			this.query = query;
-			this.query.Changed += HandleChanged;
-
-			Reload ();
-		}
+		public DirectoryAdaptor (PhotoQuery query, bool order_ascending)
+			: base (query, order_ascending)
+		{ }
 	}
 }
