@@ -21,7 +21,11 @@ namespace FSpot.Editors {
 
 		public Tilt (PhotoImageView view) : base (view)
 		{
-			name = Catalog.GetString ("Straighten");
+		}
+
+		protected override string GetTitle ()
+		{
+			return Catalog.GetString ("Straighten");
 		}
 
 		protected override void SetView (PhotoImageView view)
@@ -42,10 +46,10 @@ namespace FSpot.Editors {
 			box.PackStart (scale);
 			HBox actions = new HBox ();
 			actions.Spacing = 12;
-			Button cancel = new Button ("Cancel");
+			Button cancel = new Button (Stock.Cancel);
 			cancel.Clicked += HandleCancel;
 			actions.PackStart (cancel);
-			Button apply = new Button ("Apply");
+			Button apply = new Button (Stock.Apply);
 			apply.Clicked += HandleApply;
 			actions.PackStart (apply);
 			box.PackStart (actions);
