@@ -99,6 +99,10 @@ public class GPhotoCamera
 	{
 		if (camera_fs == null) 
 			throw new InvalidOperationException ();
+
+		//workaround for nikon dslr in ptp mode
+		if (dir == "/special/")
+			return;
 		
 		//files
 		CameraList filelist = camera_fs.ListFiles(dir, context);
@@ -247,7 +251,7 @@ public class GPhotoCamera
 }
 
 	
-public class GPhotoCameraFile
+public class GPhotoCameraFile : IComparable
 {
 	string directory;
 	string filename;
@@ -293,7 +297,7 @@ public class GPhotoCameraFile
 		}
 	}
 	
-	public void ReleaseGPhotoResources()
+	public void ReleaseGPhotoResources ()
 	{
 		if (normal != null) 
 			normal.Dispose ();
@@ -301,4 +305,13 @@ public class GPhotoCameraFile
 		if (preview != null) 
 			preview.Dispose ();
 	}
+
+	public int CompareTo (object obj)
+	{
+		GPhotoCameraFile f2 = obj as GPhotoCameraFile;
+		int result = Directory.CompareTo (f2.Directory);
+		if (result == 0)
+			result = FileName.CompareTo (f2.FileName);
+		return result;
+	}
 }
