@@ -26,7 +26,7 @@ using FSpot;
 using FSpot.Utils;
 using FSpot.UI.Dialog;
 
-public class ImportCommand : FSpot.GladeDialog
+public class ImportCommand : GladeDialog
 {
 	internal class SourceItem : ImageMenuItem
 	{
@@ -624,7 +624,7 @@ public class ImportCommand : FSpot.GladeDialog
 		GtkUtil.ModifyColors (photo_scrolled);
 		GtkUtil.ModifyColors (photo_view);
 
-		photo_view.Pixbuf = FSpot.Global.IconTheme.LoadIcon ("f-spot", 48, (Gtk.IconLookupFlags)0);
+		photo_view.Pixbuf = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, "f-spot", 48, (Gtk.IconLookupFlags)0);
 		photo_view.Fit = true;
 			
 		tag_entry = new FSpot.Widgets.TagEntry (MainWindow.Toplevel.Database.Tags, false);
@@ -720,6 +720,7 @@ public class ImportCommand : FSpot.GladeDialog
 		foreach (string tagname in new_tags) {
 			Tag t = db.Tags.GetTagByName (tagname);
 			if (t == null) {
+				// Note for translators: 'Import Tags' is no command, it means 'Tags used in Import'
 				Category default_category = db.Tags.GetTagByName (Catalog.GetString ("Import Tags")) as Category;
 				if (default_category == null) {
 					default_category = db.Tags.CreateCategory (null, Catalog.GetString ("Import Tags"));
