@@ -1,9 +1,12 @@
+using Cairo;
+
 using Gtk;
+using Gdk;
 using System;
 using System.Runtime.InteropServices;
-using Cairo;
 using Mono.Unix;
 using FSpot.Widgets;
+using FSpot.UI.Dialog;
 
 namespace FSpot {
 	public class Sharpener : Loupe {
@@ -301,14 +304,23 @@ namespace FSpot {
 							    Allocation.Width, 
 							    Allocation.Height, 1);
 			
+#if CAIRO_1_2_5
+			Context g = CairoHelper.Create (bitmap);
+#else			
 			Context g = CairoUtils.CreateContext (bitmap);
+#endif			
 			DrawShape (g, Allocation.Width, Allocation.Height);
+			
 			((IDisposable)g).Dispose ();
 
 			if (use_shape_ext)
 				ShapeCombineMask (bitmap, 0, 0);
 			else {
+#if CAIRO_1_2_5			 
+				Cairo.Context rgba = CairoHelper.Create (GdkWindow);
+#else				
 				Context rgba = CairoUtils.CreateContext (GdkWindow);
+#endif				
 				DrawShape (rgba, Allocation.Width, Allocation.Height);
 				((IDisposable)rgba).Dispose ();
 				try {
@@ -339,13 +351,13 @@ namespace FSpot {
 			hotspot.X = (int) Math.Ceiling (Center.X + x_proj);
 			hotspot.Y = (int) Math.Ceiling (Center.Y + y_proj);
 		}
-		
-		private void DrawShape (Cairo.Context g, int width, int height)
+	
+		private void DrawShape (Context g, int width, int height)
 		{
 			int inner_x = radius + border + inner;
 			int cx = Center.X;
 			int cy = Center.Y;
-			
+		
 			g.Operator = Operator.Source;
 			g.Source = new SolidPattern (new Cairo.Color (0,0,0,0));
 			g.Rectangle (0, 0, width, height);
@@ -354,7 +366,7 @@ namespace FSpot {
 			g.NewPath ();
 			g.Translate (cx, cy);
 			g.Rotate (angle);
-			
+
 			g.Source = new SolidPattern (new Cairo.Color (0.2, 0.2, 0.2, .6));
 			g.Operator = Operator.Over;
 			g.Rectangle (0, - (border + inner), inner_x, 2 * (border + inner));
@@ -365,7 +377,7 @@ namespace FSpot {
 			g.Source = new SolidPattern (new Cairo.Color (0, 0, 0, 1.0));
 			g.Operator = Operator.DestOut;
 			g.Arc (inner_x, 0, inner, 0, 2 * Math.PI);
-#if true
+#if true			
 			g.Fill ();
 #else
 			g.FillPreserve ();
@@ -383,13 +395,22 @@ namespace FSpot {
 			g.Matrix = new Matrix ();
 			g.Translate (cx, cy);
 			if (source != null)
-				SetSourcePixbuf (g, source, -source.Width / 2, -source.Height / 2);
+#if CAIRO_1_2_5			 
+			CairoHelper.SetSourcePixbuf (g, source, -source.Width / 2, -source.Height / 2);
+#else
+			SetSourcePixbuf (g, source, -source.Width / 2, -source.Height / 2);
+#endif								
 
 			g.Arc (0, 0, radius, 0, 2 * Math.PI);
 			g.Fill ();
 
 			if (overlay != null) {
+#if CAIRO_1_2_5			 
+				CairoHelper.SetSourcePixbuf (g, overlay, -overlay.Width / 2, -overlay.Height / 2);
+#else
 				SetSourcePixbuf (g, overlay, -overlay.Width / 2, -overlay.Height / 2);
+
+#endif
 				g.Arc (0, 0, radius, angle, angle + Math.PI);
 				g.ClosePath ();
 				g.FillPreserve ();
@@ -398,9 +419,28 @@ namespace FSpot {
 			}
 		}
 
+#if !CAIRO_1_2_5
+		[DllImport("libgdk-2.0-0.dll")] 	 
+	        extern static void gdk_cairo_set_source_pixbuf (IntPtr handle, 	 
+	                                                        IntPtr pixbuf, 	 
+	                                                        double        pixbuf_x, 	 
+	                                                        double        pixbuf_y); 	 
+	  	 
+	        [Obsolete ("use Gdk.CairoHelper.SetSourcePixbuf instead")] 	 
+	        static void SetSourcePixbuf (Context ctx, Gdk.Pixbuf pixbuf, double x, double y) 	 
+	        { 	 
+	                gdk_cairo_set_source_pixbuf (ctx.Handle, pixbuf.Handle, x, y); 	 
+	        } 	 
+#endif				
+
 		protected override bool OnExposeEvent (Gdk.EventExpose args)
 		{
-			Context g = CairoUtils.CreateContext (GdkWindow);
+#if CAIRO_1_2_5		 
+			Context g = CairoHelper.Create (GdkWindow);
+#else
+			Context g = CairoUtils.CreateContext (GdkWindow);			
+#endif						
+			
 			DrawShape (g, Allocation.Width, Allocation.Height);
 			//base.OnExposeEvent (args);
 			((IDisposable)g).Dispose ();
@@ -408,6 +448,15 @@ namespace FSpot {
 
 		}
 		
+                [DllImport ("libcairo-2.dll")]
+                static extern void cairo_user_to_device (IntPtr cr, ref double x, ref double y);
+
+		static void UserToDevice (Context ctx, ref double x, ref double y)
+		{
+			cairo_user_to_device (ctx.Handle, ref x, ref y);
+		}
+
+		
 		bool dragging = false;
 		bool rotate = false;
 		Delay drag;
@@ -556,31 +605,10 @@ namespace FSpot {
 			return widget;
 		}
 		
-		
-                [DllImport ("libcairo-2.dll")]
-                static extern void cairo_user_to_device (IntPtr cr, ref double x, ref double y);
-
-		static void UserToDevice (Context ctx, ref double x, ref double y)
-		{
-			cairo_user_to_device (ctx.Handle, ref x, ref y);
-		}
-
-		[DllImport("libgdk-2.0-0.dll")]
-		extern static void gdk_cairo_set_source_pixbuf (IntPtr handle,
-								IntPtr pixbuf,
-								double        pixbuf_x,
-								double        pixbuf_y);
-
-		static void SetSourcePixbuf (Context ctx, Gdk.Pixbuf pixbuf, double x, double y)
-		{
-			gdk_cairo_set_source_pixbuf (ctx.Handle, pixbuf.Handle, x, y);
-		}
-
 		protected override void OnSizeRequested (ref Requisition requisition) 
 		{
 			Layout ();
 			requisition = Bounds;
-			base.OnSizeRequested (ref requisition);
 		}
 
 		protected virtual void BuildUI ()
