@@ -2,7 +2,6 @@ using Gdk;
 using Gtk;
 using GtkSharp;
 using Glade;
-using Gnome;
 using Mono.Unix;
 using System;
 using System.Text;
@@ -16,7 +15,11 @@ using System.Web.Mail;
 using Banshee.Kernel;
 
 using FSpot;
+using FSpot.Query;
 using FSpot.Widgets;
+using FSpot.Utils;
+using FSpot.UI.Dialog;
+
 using LibGPhoto2;
 
 public class MainWindow {
@@ -77,8 +80,10 @@ public class MainWindow {
 	[Glade.Widget] CheckMenuItem display_toolbar;
 	[Glade.Widget] CheckMenuItem display_sidebar;
 	[Glade.Widget] CheckMenuItem display_timeline;
+	[Glade.Widget] CheckMenuItem display_filmstrip;
 	[Glade.Widget] CheckMenuItem display_dates_menu_item;
 	[Glade.Widget] CheckMenuItem display_tags_menu_item;
+	[Glade.Widget] CheckMenuItem display_ratings_menu_item;
 
 	[Glade.Widget] MenuItem zoom_in;
 	[Glade.Widget] MenuItem zoom_out;
@@ -98,6 +103,7 @@ public class MainWindow {
 	[Glade.Widget] MenuItem find_add_tag_with;
 	
 	[Glade.Widget] MenuItem clear_date_range;
+	[Glade.Widget] MenuItem clear_rating_filter;
 
 	[Glade.Widget] CheckMenuItem find_untagged;
 	
@@ -320,7 +326,7 @@ public class MainWindow {
 			query = new FSpot.PhotoQuery (db.Photos);
 		} catch (System.Exception e) {
 			//FIXME assume any exception here is due to a corrupt db and handle that.
-			RestoreDb (e);
+			new RepairDbDialog (e, db.Repair (), main_window);
 			query = new FSpot.PhotoQuery (db.Photos);
 		}
 
@@ -338,7 +344,7 @@ public class MainWindow {
 #endif
 
 		group_selector = new FSpot.GroupSelector ();
-		group_selector.Adaptor = new FSpot.TimeAdaptor (query);
+		group_selector.Adaptor = new FSpot.TimeAdaptor (query, (bool)Preferences.Get (Preferences.GROUP_ADAPTOR_ORDER_ASC));
 
 		group_selector.ShowAll ();
 		
@@ -364,6 +370,7 @@ public class MainWindow {
 		LoadPreference (Preferences.ZOOM);
 		LoadPreference (Preferences.SHOW_TAGS);
 		LoadPreference (Preferences.SHOW_DATES);
+		LoadPreference (Preferences.SHOW_RATINGS);
 		icon_view_scrolled.Add (icon_view);
 		icon_view.DoubleClicked += HandleDoubleClicked;
 		icon_view.Vadjustment.ValueChanged += HandleIconViewScroll;
@@ -433,24 +440,8 @@ public class MainWindow {
 		this.selection.Changed += HandleSelectionChanged;
 		this.selection.ItemsChanged += HandleSelectionItemsChanged;
 
-		try {
-			if (export.Submenu != null)
-				export.RemoveSubmenu ();
-
-			export.Submenu = (Mono.Addins.AddinManager.GetExtensionNode ("/FSpot/Menus/Exports") as FSpot.Extensions.SubmenuNode).GetMenuItem ().Submenu;
-		} catch {
-			Console.WriteLine ("There's (maybe) something wrong with some of the installed extensions. You can try removing the directory addin-db-000 from ~/.gnome2/f-spot/");
-		}
-
-		try {
-			if (tools.Submenu != null)
-				tools.RemoveSubmenu ();
-
-			tools.Submenu = (Mono.Addins.AddinManager.GetExtensionNode ("/FSpot/Menus/Tools") as FSpot.Extensions.SubmenuNode).GetMenuItem ().Submenu;
-		} catch {
-			Console.WriteLine ("There's (maybe) something wrong with some of the installed extensions. You can try removing the directory addin-db-000 from ~/.gnome2/f-spot/");
-			tools.Visible = false;
-		}
+		Mono.Addins.AddinManager.ExtensionChanged += PopulateExtendableMenus;
+		PopulateExtendableMenus (null, null);
 
 		UpdateMenus ();
 
@@ -464,6 +455,7 @@ public class MainWindow {
 		LoadPreference (Preferences.SHOW_TOOLBAR);
 		LoadPreference (Preferences.SHOW_SIDEBAR);
 		LoadPreference (Preferences.SHOW_TIMELINE);
+		LoadPreference (Preferences.SHOW_FILMSTRIP);
 
 		LoadPreference (Preferences.GNOME_MAILTO_ENABLED);
 		
@@ -518,7 +510,11 @@ public class MainWindow {
 		case ModeType.IconView:
 			if (view_notebook.CurrentPage != 0)
 				view_notebook.CurrentPage = 0;
-				
+
+			display_timeline.Sensitive = true;
+			display_filmstrip.Sensitive = false;
+			group_selector.Visible = display_timeline.Active;
+
 			ColorDialog.Close ();
 			if (photo_view.View.Loupe != null)
 				photo_view.View.Loupe.Destroy ();
@@ -528,6 +524,10 @@ public class MainWindow {
 		case ModeType.PhotoView:
 			if (view_notebook.CurrentPage != 1)
 				view_notebook.CurrentPage = 1;
+
+			display_timeline.Sensitive = false;
+			display_filmstrip.Sensitive = true;
+			group_selector.Visible = false;
 			
 			JumpTo (icon_view.FocusCell);
 			zoom_scale.Value = photo_view.NormalizedZoom;
@@ -702,7 +702,7 @@ public class MainWindow {
 			}
 		}
 
-		private void HandleQueryItemsChanged (IBrowsableCollection collection, BrowsableArgs args)
+		private void HandleQueryItemsChanged (IBrowsableCollection collection, BrowsableEventArgs args)
 		{
 			// FIXME for now we only listen to changes directly from the query
 			// when we are in PhotoView mode because we presume that we'll get
@@ -712,7 +712,7 @@ public class MainWindow {
 
 			foreach (int item in args.Items) {
 				if (win.photo_view.Item.Index == item ) {
-					ItemsChanged (this, new BrowsableArgs (0));
+					ItemsChanged (this, new BrowsableEventArgs (0));
 					break;
 				}
 			}
@@ -732,7 +732,7 @@ public class MainWindow {
 
 		}
 
-		private void HandleSelectionItemsChanged (IBrowsableCollection collection,  BrowsableArgs args)
+		private void HandleSelectionItemsChanged (IBrowsableCollection collection,  BrowsableEventArgs args)
 		{
 			if (win.view_mode == ModeType.IconView && ItemsChanged != null)
 				ItemsChanged (this, args);
@@ -753,7 +753,7 @@ public class MainWindow {
 		UpdateStatusLabel();	
 	}
 
-	private void HandleSelectionItemsChanged (IBrowsableCollection collection, BrowsableArgs args)
+	private void HandleSelectionItemsChanged (IBrowsableCollection collection, BrowsableEventArgs args)
 	{
 		UpdateMenus ();
 		UpdateTagEntryFromSelection ();
@@ -781,6 +781,15 @@ public class MainWindow {
 		return SelectedPhotos (SelectedIds ());
 	}
 
+	public Photo [] ActivePhotos () 
+	{
+		return query.Photos;
+	}
+
+	public PhotoQuery Query {
+		get { return query; }
+	}
+
 	//
 	// Change Notification functions
 	//
@@ -861,20 +870,6 @@ public class MainWindow {
 		args.RetVal = true;
 	}
 
-	void RestoreDb (System.Exception e)
-	{
-		string backup = db.Repair ();
-		string short_msg = Catalog.GetString ("Error loading database.");
-		string long_msg = Catalog.GetString ("F-Spot encountered an error while loading the photo database. " + 
-								"The old database has be moved to {0} and a new database has been created.");
-
-		HigMessageDialog md = new HigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
-							    MessageType.Error, ButtonsType.Ok, 
-							    short_msg, String.Format (long_msg, backup));
-		md.Run ();
-		md.Destroy ();
-	}
-
 	void HandleTagSelectionDragBegin (object sender, DragBeginArgs args)
 	{
 		Tag [] tags = tag_selection_widget.TagHighlight;
@@ -1196,10 +1191,50 @@ public class MainWindow {
 
 	void HandleIconViewKeyPressEvent (object sender, Gtk.KeyPressEventArgs args)
 	{
-		if (args.Event.Key == Gdk.Key.Delete) {
+		bool alt = ModifierType.Mod1Mask == (args.Event.State & ModifierType.Mod1Mask);
+
+		switch (args.Event.Key) {
+		case Gdk.Key.Delete:
 			HandleRemoveCommand (sender, (EventArgs) args);
+			args.RetVal = true;
+			break;
+		case Gdk.Key.Key_0:
+			if (alt) {
+				HandleRatingMenuSelected (0);
+				args.RetVal = true;
+			}
+			break;
+		case Gdk.Key.Key_1:
+			if (alt) {
+				HandleRatingMenuSelected (1);
+				args.RetVal = true;
+			}
+			break;
+		case Gdk.Key.Key_2:
+			if (alt) {
+				HandleRatingMenuSelected (2);
+				args.RetVal = true;
+			}
+			break;
+		case Gdk.Key.Key_3:
+			if (alt) {
+				HandleRatingMenuSelected (3);
+				args.RetVal = true;
+			}
+			break;
+		case Gdk.Key.Key_4:
+			if (alt) {
+				HandleRatingMenuSelected (4);
+				args.RetVal = true;
+			}
+			break;
+		case Gdk.Key.Key_5:
+			if (alt) {
+				HandleRatingMenuSelected (5);
+				args.RetVal = true;
+			}
+			break;
 		}
-		UpdateStatusLabel();		
 	}
 
 	public void ImportUriList (UriList list, bool copy) 
@@ -1297,7 +1332,7 @@ public class MainWindow {
 	// IconView event handlers
 	// 
 
-	void HandleDoubleClicked (IconView icon_view, int clicked_item)
+	void HandleDoubleClicked (FSpot.Widgets.IconView icon_view, int clicked_item)
 	{
 		icon_view.FocusCell = clicked_item;
 		SetViewMode (ModeType.PhotoView);
@@ -1309,6 +1344,7 @@ public class MainWindow {
 	
 	void HandlePhotoViewKeyPressEvent (object sender, Gtk.KeyPressEventArgs args)
 	{
+		bool alt = ModifierType.Mod1Mask == (args.Event.State & ModifierType.Mod1Mask);
 		switch (args.Event.Key) {
 		case Gdk.Key.F:
 		case Gdk.Key.f:
@@ -1323,6 +1359,42 @@ public class MainWindow {
 			HandleRemoveCommand (sender, (EventArgs) args);
 			args.RetVal = true;
 			break;
+		case Gdk.Key.Key_0:
+			if (alt) {
+				HandleRatingMenuSelected (0);
+				args.RetVal = true;
+			}
+			break;
+		case Gdk.Key.Key_1:
+			if (alt) {
+				HandleRatingMenuSelected (1);
+				args.RetVal = true;
+			}
+			break;
+		case Gdk.Key.Key_2:
+			if (alt) {
+				HandleRatingMenuSelected (2);
+				args.RetVal = true;
+			}
+			break;
+		case Gdk.Key.Key_3:
+			if (alt) {
+				HandleRatingMenuSelected (3);
+				args.RetVal = true;
+			}
+			break;
+		case Gdk.Key.Key_4:
+			if (alt) {
+				HandleRatingMenuSelected (4);
+				args.RetVal = true;
+			}
+			break;
+		case Gdk.Key.Key_5:
+			if (alt) {
+				HandleRatingMenuSelected (5);
+				args.RetVal = true;
+			}
+			break;
 		default:
 			break;
 		}
@@ -1380,6 +1452,27 @@ public class MainWindow {
 		Gtk.Drag.Finish (args.Context, true, false, args.Time);
 	}	
 
+	//
+	// RatingMenu commands
+	//
+	
+	public void HandleRatingMenuSelected (int r) 
+	{
+		if (view_mode == ModeType.PhotoView)
+			this.photo_view.UpdateRating(r);
+
+		uint timer = Log.DebugTimerStart ();
+		Photo p;
+		db.BeginTransaction ();
+		foreach (int num in SelectedIds ()) {
+			p = query.Photos [num];
+			p.Rating = (uint) r;
+			query.Commit (num);
+		}
+		db.CommitTransaction ();
+		Log.DebugTimerPrint (timer, "HandleRating took {0}");
+	}
+
 	//
 	// TagMenu commands.
 	//
@@ -1516,7 +1609,12 @@ public class MainWindow {
 	
 	void HandlePrintCommand (object sender, EventArgs e)
 	{
+#if !GTK_2_10
 		new FSpot.PrintDialog (SelectedPhotos ());
+#else
+		FSpot.PrintOperation print = new FSpot.PrintOperation (SelectedPhotos ());
+		print.Run (PrintOperationAction.PrintDialog, null);
+#endif
 	}
 
 	private Gtk.Dialog info_display_window;
@@ -1561,7 +1659,7 @@ public class MainWindow {
 	void HandleViewDirectory (object sender, EventArgs args)
 	{
 		Gtk.Window win = new Gtk.Window ("Directory View");
-		IconView view = new IconView (new FSpot.DirectoryCollection (System.IO.Directory.GetCurrentDirectory ()));
+		FSpot.Widgets.IconView view = new FSpot.Widgets.IconView (new FSpot.DirectoryCollection (System.IO.Directory.GetCurrentDirectory ()));
 		new FSpot.PreviewPopup (view);
 
 		view.DisplayTags = false;
@@ -1599,67 +1697,12 @@ public class MainWindow {
 
 	public static void HandleHelp (object sender, EventArgs args)
 	{
-		GnomeUtil.ShowHelp ("f-spot.xml", null, Toplevel.Window.Screen, null);
-//		Gnome.Vfs.Url.Show("ghelp:" + 
-//				FSpot.Global.HelpDirectory +
-//				System.IO.Path.DirectorySeparatorChar +
-//				"f-spot.xml");
+		GnomeUtil.ShowHelp ("f-spot.xml", null, FSpot.Global.HelpDirectory, Toplevel.Window.Screen);
 	}
 
-	private static About about = null;
 	public static void HandleAbout (object sender, EventArgs args)
 	{
-		string [] authors = new string [] {
-			"Ettore Perazzoli",
-			"Lawrence Ewing",
-			"Laurence Hygate",
-			"Alex Graveley",
-			"Todd Berman",
-			"Ben Monnahan",
-			"Stephane Delcroix",
-			"Vladimir Vukicevic",
-			"Aaron Bockover",
-			"Bengt Thuree",
-			"Jon Trowbridge",
-			"Joe Shaw",
-			"Nat Friedman",
-			"Ruben Vermeersch",
-			"Tambet Ingo",
-			"MOREAU Vincent",
-			"Alvaro del Castillo",
-			"Lee Willis",
-			"Alessandro Gervaso",
-			"Peter Johanson",
-			"Miguel de Icaza",
-			"Chad Files",
-			"Grahm Orr",
-			"Ewen Cheslack-Postava",
-			"Gabriel Burt",
-			"Patanjali Somayaji",
-			"Matt Jones",
-			"Martin Willemoes Hansen",
-			"Joshua Tauberer",
-			"Joerg Buesse",
-			"Jakub Steiner",
-			"Xavier Bouchoux"
-		};
-
-                // Translators should localize the following string
-                // * which will give them credit in the About box.
-                // * E.g. "Martin Willemoes Hansen"
-                string translators = Catalog.GetString ("translator-credits");
-                if (System.String.Compare (translators, "translator-credits") == 0) {
-                	translators = null;
-                }
-
-		if (about == null)
-			about = new About ("F-Spot", 
-			   FSpot.Defines.VERSION, 
-			   Catalog.GetString ("Copyright \x00a9 2003-2007 Novell Inc."),
-                           null, authors, new string [0], translators, 
-			   PixbufUtils.LoadFromAssembly ("f-spot-logo.svg"));
-		about.Destroyed += delegate (object o, EventArgs e) {about = null;};
-		about.Show ();
+		FSpot.UI.Dialog.AboutDialog.ShowUp ();
 	}
 
 	void HandleTagSizeChange (object sender, EventArgs args)
@@ -1698,7 +1741,7 @@ public class MainWindow {
 
 		group_selector.Adaptor.GlassSet -= HandleAdaptorGlassSet;
 		group_selector.Adaptor.Changed -= HandleAdaptorChanged;
-		group_selector.Adaptor = new FSpot.TimeAdaptor (query);
+		group_selector.Adaptor = new FSpot.TimeAdaptor (query, reverse_order.Active);
 
 		group_selector.Mode = FSpot.GroupSelector.RangeType.Min;
 		group_selector.Adaptor.GlassSet += HandleAdaptorGlassSet;
@@ -1719,7 +1762,7 @@ public class MainWindow {
 
 		group_selector.Adaptor.GlassSet -= HandleAdaptorGlassSet;
 		group_selector.Adaptor.Changed -= HandleAdaptorChanged;
-		group_selector.Adaptor = new FSpot.DirectoryAdaptor (query); 	
+		group_selector.Adaptor = new FSpot.DirectoryAdaptor (query, reverse_order.Active); 	
 
 		group_selector.Mode = FSpot.GroupSelector.RangeType.Min;
 		group_selector.Adaptor.GlassSet += HandleAdaptorGlassSet;
@@ -1781,9 +1824,11 @@ public class MainWindow {
 
 		Preferences.Set (Preferences.SHOW_TOOLBAR,		toolbar.Visible);
 		Preferences.Set (Preferences.SHOW_SIDEBAR,		info_vbox.Visible);
-		Preferences.Set (Preferences.SHOW_TIMELINE,		group_selector.Visible);
+		Preferences.Set (Preferences.SHOW_TIMELINE,		display_timeline.Active);
+		Preferences.Set (Preferences.SHOW_FILMSTRIP,		display_filmstrip.Active);
 		Preferences.Set (Preferences.SHOW_TAGS,			icon_view.DisplayTags);
 		Preferences.Set (Preferences.SHOW_DATES,		icon_view.DisplayDates);
+		Preferences.Set (Preferences.SHOW_RATINGS,		icon_view.DisplayRatings);
 
 		Preferences.Set (Preferences.GROUP_ADAPTOR,		(group_selector.Adaptor is DirectoryAdaptor) ? 1 : 0);
 		Preferences.Set (Preferences.GROUP_ADAPTOR_ORDER_ASC,   group_selector.Adaptor.OrderAscending);
@@ -1997,64 +2042,13 @@ public class MainWindow {
 
 	void HandleSharpen (object sender, EventArgs args)
 	{
-		Gtk.Dialog dialog = new Gtk.Dialog (Catalog.GetString ("Unsharp Mask"), main_window, Gtk.DialogFlags.Modal);
-
-		dialog.VBox.Spacing = 6;
-		dialog.VBox.BorderWidth = 12;
-		
-		Gtk.Table table = new Gtk.Table (3, 2, false);
-		table.ColumnSpacing = 6;
-		table.RowSpacing = 6;
-		
-		table.Attach (new Gtk.Label (Catalog.GetString ("Amount:")), 0, 1, 0, 1);
-		table.Attach (new Gtk.Label (Catalog.GetString ("Radius:")), 0, 1, 1, 2);
-		table.Attach (new Gtk.Label (Catalog.GetString ("Threshold:")), 0, 1, 2, 3);
-
-		Gtk.SpinButton amount_spin = new Gtk.SpinButton (0.5, 100.0, .01);
-		Gtk.SpinButton radius_spin = new Gtk.SpinButton (5.0, 50.0, .01);
-		Gtk.SpinButton threshold_spin = new Gtk.SpinButton (0.0, 50.0, .01);
-
-		table.Attach (amount_spin, 1, 2, 0, 1);
-		table.Attach (radius_spin, 1, 2, 1, 2);
-		table.Attach (threshold_spin, 1, 2, 2, 3);
-		
-		dialog.VBox.PackStart (table);
-
-		dialog.AddButton (Gtk.Stock.Cancel, Gtk.ResponseType.Cancel);
-		dialog.AddButton (Gtk.Stock.Ok, Gtk.ResponseType.Ok);
-
-		dialog.ShowAll ();
-
-		Gtk.ResponseType response = (Gtk.ResponseType) dialog.Run ();
-
-		if (response == Gtk.ResponseType.Ok) {
-			foreach (int id in SelectedIds ()) {
-				Photo photo = query.Photos [id];
-				try {
-					Gdk.Pixbuf orig = FSpot.PhotoLoader.Load (query, id);
-					Gdk.Pixbuf final = PixbufUtils.UnsharpMask (orig, radius_spin.Value, amount_spin.Value, threshold_spin.Value);
-					
-					bool create_version = photo.DefaultVersion.IsProtected;
-
-					photo.SaveVersion (final, create_version);
-					query.Commit (id);
-				} catch (System.Exception e) {
-					string msg = Catalog.GetString ("Error saving sharpened photo");
-					string desc = String.Format (Catalog.GetString ("Received exception \"{0}\". Unable to save photo {1}"),
-								     e.Message, photo.Name.Replace ("_", "__"));
-					
-					HigMessageDialog md = new HigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
-										    Gtk.MessageType.Error, ButtonsType.Ok, 
-										    msg,
-										    desc);
-					md.Run ();
-					md.Destroy ();
-				}
-			
-			}
+		// Don't steal characters from any text entries
+		if (Window.Focus is Gtk.Entry && Gtk.Global.CurrentEvent is Gdk.EventKey) {
+			Window.Focus.ProcessEvent (Gtk.Global.CurrentEvent);
+			return;
 		}
-
-		dialog.Destroy ();
+		
+		photo_view.View.ShowSharpener ();
 	}
 
 	void HandleDisplayToolbar (object sender, EventArgs args)
@@ -2078,6 +2072,11 @@ public class MainWindow {
 		icon_view.DisplayDates = display_dates_menu_item.Active;
 	}
 
+	void HandleDisplayRatings (object sender, EventArgs args)
+	{
+		icon_view.DisplayRatings = display_ratings_menu_item.Active;
+	}
+
 	void HandleDisplayGroupSelector (object sender, EventArgs args)
 	{
 		if (group_selector.Visible)
@@ -2086,6 +2085,11 @@ public class MainWindow {
 			group_selector.Show ();
 	}
 
+	void HandleDisplayFilmstrip (object sender, EventArgs args)
+	{
+		photo_view.FilmStripVisibility = display_filmstrip.Active;
+	}
+
 	void HandleDisplayInfoSidebar (object sender, EventArgs args)
 	{
 		if (info_vbox.Visible)
@@ -2096,18 +2100,7 @@ public class MainWindow {
 
 	void HandleViewSlideShow (object sender, EventArgs args)
 	{
-		int active = Math.Max (ActiveIndex (), 0);
-		if (fsview == null) {
-			fsview = new FSpot.FullScreenView (query);
-			fsview.Destroyed += HandleFullScreenViewDestroy;
-			fsview.View.Item.Index = active;
-			ColorDialog.SwitchViews (fsview.View);
-		} else {
-			// FIXME this needs to be another mode like PhotoView and IconView mode.
-			fsview.View.Item.Index = active;
-		}
-		
-		fsview.Show ();
+		HandleViewFullscreen (sender, args);
 		fsview.PlayPause ();
 	}
 
@@ -2139,9 +2132,9 @@ public class MainWindow {
 
 	void HandleViewFullscreen (object sender, EventArgs args)
 	{
-		int active = Math.Max (ActiveIndex (), 0);
+		int active = (selection.Count > 0 ? SelectedIds() [0] : 0);
 		if (fsview == null) {
-			fsview = new FSpot.FullScreenView (query);
+			fsview = new FSpot.FullScreenView (query, main_window);
 			fsview.Destroyed += HandleFullScreenViewDestroy;
 			fsview.View.Item.Index = active;
 			ColorDialog.SwitchViews (fsview.View);
@@ -2186,39 +2179,25 @@ public class MainWindow {
 	{
 		if (find_untagged.Active != query.Untagged)
 			find_untagged.Active = query.Untagged;
-
 		
 		clear_date_range.Sensitive = (query.Range != null);
+		clear_rating_filter.Sensitive = (query.RatingRange != null);
 		UpdateStatusLabel ();
 	}
 
 	private void UpdateStatusLabel ()
 	{
-		StringBuilder label_str = new StringBuilder();
-		
-		switch (view_mode) {
-		case ModeType.PhotoView:
-			break;
-		case ModeType.IconView:		
-			label_str.Append (String.Format (Catalog.GetPluralString ("{0} Photo", 
-										  "{0} Photos", 
-										  query.Count), 
-							 query.Count));
-
-			int total_photos = Database.Photos.TotalPhotos;
-			if (total_photos != query.Count)
-				label_str.Append (String.Format (Catalog.GetString (" out of {0}"), total_photos));
-
-			if ((selection != null) && (selection.Count > 0))
-				label_str.Append (String.Format (Catalog.GetPluralString (" ({0} selected)", 
-											  " ({0} selected)", 
-											  selection.Count), 
-								 selection.Count));
-			break;
-		}
-		
-		status_label.Text = label_str.ToString();
+		//uint timer = Log.DebugTimerStart ();
+		int total_photos = Database.Photos.TotalPhotos;
+		if (total_photos != query.Count)
+			status_label.Text = String.Format (Catalog.GetPluralString ("{0} Photo out of {1}", "{0} Photos out of {1}", query.Count), query.Count, total_photos);
+		else
+			status_label.Text = String.Format (Catalog.GetPluralString ("{0} Photo", "{0} Photos", query.Count), query.Count);
+	
+		if ((selection != null) && (selection.Count > 0))
+			status_label.Text += String.Format (Catalog.GetPluralString (" ({0} selected)", " ({0} selected)", selection.Count), selection.Count);
 		status_label.UseMarkup = true;
+		//Log.DebugTimerPrint (timer, "UpdateStatusLabel took {0}");
 	}
 	
 	void HandleZoomChanged (object sender, System.EventArgs args)
@@ -2431,6 +2410,12 @@ public class MainWindow {
 		Tag [] tags = this.tag_selection_widget.TagHighlight;
 
 		System.Array.Sort (tags, new TagRemoveComparer ());
+	
+		//How many pictures are associated to these tags?
+		Db db = MainWindow.Toplevel.Database;
+		FSpot.PhotoQuery count_query = new FSpot.PhotoQuery(db.Photos);
+		count_query.Terms = FSpot.OrTerm.FromTags(tags);
+		int associated_photos = count_query.Photos.Length;
 
 		string header;
 		if (tags.Length == 1)
@@ -2439,7 +2424,15 @@ public class MainWindow {
 			header = String.Format (Catalog.GetString ("Delete the {0} selected tags?"), tags.Length);
 		
 		header = String.Format (header, tags.Length);
-		string msg = Catalog.GetString("If you delete a tag, all associations with photos are lost.");
+		string msg = String.Empty;
+		if (associated_photos > 0) {
+			string photodesc = Catalog.GetPluralString ("photo", "photos", associated_photos);
+			msg = String.Format( 
+				Catalog.GetPluralString("If you delete this tag, the association with {0} {1} will be lost.",
+							"If you delete these tags, the association with {0} {1} will be lost.",
+							tags.Length),
+				associated_photos, photodesc);
+		}
 		string ok_caption = Catalog.GetPluralString ("_Delete tag", "_Delete tags", tags.Length);
 		
 		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(main_window, 
@@ -2527,12 +2520,14 @@ public class MainWindow {
 
 	void HandleSetAsBackgroundCommand (object sender, EventArgs args)
 	{
+#if !NOGCONF
 		Photo current = CurrentPhoto;
 
 		if (current == null)
 			return;
 
-		Preferences.SetAsBackground (current.DefaultVersionUri.LocalPath);
+		GnomeUtil.SetBackgroundImage (current.DefaultVersionUri.LocalPath);
+#endif
 	}
 
 	void HandleSetDateRange (object sender, EventArgs args) {
@@ -2562,7 +2557,15 @@ public class MainWindow {
 		query.RollSet = null;
 	}
 
-	
+	void HandleSetRatingFilter (object sender, EventArgs args) {
+		RatingFilter.Set set_command = new RatingFilter.Set (query, main_window);
+		set_command.Execute ();
+	}
+
+	public void HandleClearRatingFilter (object sender, EventArgs args) {
+		query.RatingRange = null;
+	}
+
 	void HandleFindUntagged (object sender, EventArgs args) {
 		if (query.Untagged == find_untagged.Active)
 			return;
@@ -2570,7 +2573,7 @@ public class MainWindow {
 		query.Untagged = !query.Untagged;
 	}
 	
-	void OnPreferencesChanged (object sender, GConf.NotifyEventArgs args)
+	void OnPreferencesChanged (object sender, NotifyEventArgs args)
 	{
 		LoadPreference (args.Key);
 	}
@@ -2620,6 +2623,11 @@ public class MainWindow {
 				display_timeline.Active = (bool) val;
 			break;
 		
+		case Preferences.SHOW_FILMSTRIP:
+			if (display_filmstrip.Active != (bool) val)
+				display_filmstrip.Active = (bool) val;
+			break;
+		
 		case Preferences.SHOW_TAGS:
 			if (display_tags_menu_item.Active != (bool) val)
 				display_tags_menu_item.Active = (bool) val;
@@ -2631,6 +2639,11 @@ public class MainWindow {
 				//display_dates_menu_item.Toggle ();
 			break;
 		
+		case Preferences.SHOW_RATINGS:
+			if (display_ratings_menu_item.Active != (bool) val)
+				display_ratings_menu_item.Active = (bool) val;
+			break;
+		
 		case Preferences.GROUP_ADAPTOR:
 			if ((int) val == 1)
 				directory.Active = true;
@@ -2638,6 +2651,7 @@ public class MainWindow {
 
 		case Preferences.GROUP_ADAPTOR_ORDER_ASC:
 			group_selector.Adaptor.OrderAscending = (bool) val;
+			reverse_order.Active = (bool) val;
 			break;
 
 		case Preferences.GLASS_POSITION:
@@ -2760,7 +2774,7 @@ public class MainWindow {
 		if (find_add_tag_with.Submenu != null)
 			find_add_tag_with.Submenu.Dispose ();
 		
-		Gtk.Menu submenu = FSpot.Query.TermMenuItem.GetSubmenu (tag_selection_widget.TagHighlight);
+		Gtk.Menu submenu = FSpot.TermMenuItem.GetSubmenu (tag_selection_widget.TagHighlight);
 		find_add_tag_with.Sensitive = (submenu != null);
 		if (submenu != null) 
 			find_add_tag_with.Submenu = submenu;	
@@ -2781,11 +2795,11 @@ public class MainWindow {
 		// account for All and separator menu items
 		item_pos -= 2;
 		
-		FSpot.Query.Term parent_term = (FSpot.Query.Term) FSpot.Query.LogicWidget.Root.SubTerms [item_pos];
+		FSpot.Term parent_term = (FSpot.Term) FSpot.LogicWidget.Root.SubTerms [item_pos];
 		
-		if (FSpot.Query.LogicWidget.Box != null) {
-			FSpot.Query.Literal after = parent_term.Last as FSpot.Query.Literal;
-			FSpot.Query.LogicWidget.Box.InsertTerm (tag_selection_widget.TagHighlight, parent_term, after);
+		if (FSpot.LogicWidget.Box != null) {
+			FSpot.Literal after = parent_term.Last as FSpot.Literal;
+			FSpot.LogicWidget.Box.InsertTerm (tag_selection_widget.TagHighlight, parent_term, after);
 		}
 	}
 	
@@ -2819,6 +2833,7 @@ public class MainWindow {
 			delete_version_menu_item.Sensitive = false;
 			rename_version_menu_item.Sensitive = false;
 
+			sharpen.Sensitive = false;
 		} else {
 			version_menu_item.Sensitive = true;
 			create_version_menu_item.Sensitive = true;
@@ -2834,6 +2849,8 @@ public class MainWindow {
 			versions_submenu = new PhotoVersionMenu (CurrentPhoto);
 			versions_submenu.VersionIdChanged += new PhotoVersionMenu.VersionIdChangedHandler (HandleVersionIdChanged);
 			version_menu_item.Submenu = versions_submenu;
+
+			sharpen.Sensitive = (view_mode == ModeType.IconView ? false : true);
 		}
 
 		set_as_background.Sensitive = single_active;
@@ -2853,9 +2870,10 @@ public class MainWindow {
 		select_none.Sensitive = active_selection;
 		copy_location.Sensitive = active_selection;
 		exif_data.Sensitive = active_selection;
-		sharpen.Sensitive = active_selection;
 		remove_from_catalog.Sensitive = active_selection;
 		
+		clear_rating_filter.Sensitive = (query.RatingRange != null);
+
 		last_roll.Sensitive = (db.Rolls.GetRolls (1).Length > 0);
 		select_rolls.Sensitive = (db.Rolls.GetRolls (2).Length > 1);
 		clear_roll_filter.Sensitive = (query.RollSet != null);
@@ -2901,20 +2919,41 @@ public class MainWindow {
 			}
 		}
 
-        //if (last_tags_selected_count != tags_selected) {
-            ((Gtk.Label)find_add_tag.Child).Text = String.Format (
-                Catalog.GetPluralString ("Find Selected Tag", "Find Selected Tags", tags_selected), tags_selected
-            );
+		//if (last_tags_selected_count != tags_selected) {
+		((Gtk.Label)find_add_tag.Child).TextWithMnemonic = String.Format (
+			Catalog.GetPluralString ("Find _Selected Tag", "Find _Selected Tags", tags_selected), tags_selected
+		);
 
-            ((Gtk.Label)find_add_tag_with.Child).Text = String.Format (
-                Catalog.GetPluralString ("Find Selected Tag With", "Find Selected Tags With", tags_selected), tags_selected
-            );
+		((Gtk.Label)find_add_tag_with.Child).TextWithMnemonic = String.Format (
+			Catalog.GetPluralString ("Find Selected Tag _With", "Find Selected Tags _With", tags_selected), tags_selected
+		);
 
-            find_add_tag.Sensitive = tag_sensitive;
-            find_add_tag_with.Sensitive = tag_sensitive && find_add_tag_with.Submenu != null;
+		find_add_tag.Sensitive = tag_sensitive;
+		find_add_tag_with.Sensitive = tag_sensitive && find_add_tag_with.Submenu != null;
 
-            //last_tags_selected_count = tags_selected;
-        //}
+		//last_tags_selected_count = tags_selected;
+		//}
+	}
+
+	void PopulateExtendableMenus (object o, EventArgs args)
+	{
+		try {
+			if (export.Submenu != null)
+				export.Submenu.Dispose ();
+			if (tools.Submenu != null)
+				tools.RemoveSubmenu ();
+
+			export.Submenu = (Mono.Addins.AddinManager.GetExtensionNode ("/FSpot/Menus/Exports") as FSpot.Extensions.SubmenuNode).GetSubmenu ();
+			export.Submenu.ShowAll ();
+
+			tools.Submenu = (Mono.Addins.AddinManager.GetExtensionNode ("/FSpot/Menus/Tools") as FSpot.Extensions.SubmenuNode).GetSubmenu ();
+			tools.Submenu.ShowAll ();
+
+			tools.Visible = (tools.Submenu as Menu).Children.Length > 0;
+		} catch {
+			Console.WriteLine ("There's (maybe) something wrong with some of the installed extensions. You can try removing the directory addin-db-000 from ~/.gnome2/f-spot/");
+			tools.Visible = false;
+		}
 	}
 
 	public void HandleOpenWith (object sender, Gnome.Vfs.MimeApplication mime_application)
@@ -3001,7 +3040,7 @@ public class MainWindow {
 
 	public void HandlePossibleTagTyping (object sender, Gtk.KeyPressEventArgs args)
 	{
-		if (tagbar.Visible && tag_entry.HasFocus)
+		if (Selection.Count == 0 || tagbar.Visible && tag_entry.HasFocus)
 			return;
 
 #if !ALLOW_TAG_TYPING_WITHOUT_HOTKEY
