@@ -4,6 +4,7 @@ using System.Runtime.InteropServices;
 using System;
 using System.IO;
 using FSpot;
+using FSpot.Utils;
 
 /**
   1        2       3      4         5            6           7          8
@@ -184,6 +185,8 @@ public class PixbufUtils {
 
 	public static Pixbuf ShallowCopy (Pixbuf pixbuf)
 	{
+		if (pixbuf == null)
+			return null;
 		Pixbuf result = new Pixbuf (pixbuf, 0, 0, pixbuf.Width, pixbuf.Height);
 		CopyThumbnailOptions (pixbuf, result);
 		return result;
@@ -349,7 +352,11 @@ public class PixbufUtils {
 	
 	public static Pixbuf TagIconFromPixbuf (Pixbuf source)
 	{
-		int size = (int) Tag.IconSize.Large;
+		return IconFromPixbuf (source, (int) Tag.IconSize.Large);
+	}
+
+	public static Pixbuf IconFromPixbuf (Pixbuf source, int size)
+	{
 		Pixbuf tmp = null;
 		Pixbuf icon = null;
 
